/*
 * dcfldd_error.h -- replacement for non-portable GNU libc's error()
 * Copyright (C) 2024 David da Silva Polverari
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef DCFLDD_ERROR_H
#define DCFLDD_ERROR_H

void dcfldd_error(int status, int errnum, const char *format, ...);

#endif /* DCFLDD_ERROR_H */
